<!DOCTYPE html>
<html>
<head>
  <meta charset="UTF-8">
  <title>Swagger UI</title>
  <link rel="icon" type="image/png" href="{{ url_for('flask-apispec.static', filename='images/favicon-32x32.png') }}" sizes="32x32" />
  <link rel="icon" type="image/png" href="{{ url_for('flask-apispec.static', filename='images/favicon-16x16.png') }}" sizes="16x16" />

  <link href="{{ url_for('flask-apispec.static', filename='css/typography.css') }}" media='screen' rel="stylesheet" type="text/css"/>
  <link href="{{ url_for('flask-apispec.static', filename='css/reset.css') }}" media='screen' rel="stylesheet" type="text/css"/>
  <link href="{{ url_for('flask-apispec.static', filename='css/screen.css') }}" media='screen' rel="stylesheet" type="text/css"/>
  <link href="{{ url_for('flask-apispec.static', filename='css/reset.css') }}" media='print' rel="stylesheet" type="text/css"/>
  <link href="{{ url_for('flask-apispec.static', filename='css/print.css') }}" media='print' rel="stylesheet" type="text/css"/>

  <script src="{{ url_for('flask-apispec.static', filename='lib/jquery-1.8.0.min.js') }}" type="text/javascript"></script>
  <script src="{{ url_for('flask-apispec.static', filename='lib/jquery.slideto.min.js') }}" type="text/javascript"></script>
  <script src="{{ url_for('flask-apispec.static', filename='lib/jquery.wiggle.min.js') }}" type="text/javascript"></script>
  <script src="{{ url_for('flask-apispec.static', filename='lib/jquery.ba-bbq.min.js') }}" type="text/javascript"></script>
  <script src="{{ url_for('flask-apispec.static', filename='lib/handlebars-2.0.0.js') }}" type="text/javascript"></script>
  <script src="{{ url_for('flask-apispec.static', filename='lib/underscore-min.js') }}" type="text/javascript"></script>
  <script src="{{ url_for('flask-apispec.static', filename='lib/backbone-min.js') }}" type="text/javascript"></script>
  <script src="{{ url_for('flask-apispec.static', filename='swagger-ui.js') }}" type="text/javascript"></script>
  <script src="{{ url_for('flask-apispec.static', filename='lib/highlight.7.3.pack.js') }}" type="text/javascript"></script>
  <script src="{{ url_for('flask-apispec.static', filename='lib/marked.js') }}" type="text/javascript"></script>
  <script src="{{ url_for('flask-apispec.static', filename='lib/swagger-oauth.js') }}" type="text/javascript"></script>

  <script type="text/javascript">
    $(function () {
      window.swaggerUi = new SwaggerUi({
        url: '{{ url_for('flask-apispec.swagger-json') }}',
        dom_id: 'swagger-ui-container',
        supportedSubmitMethods: ['get', 'post', 'put', 'delete', 'patch'],
        onComplete: function(swaggerApi, swaggerUi) {
          $('pre code').each(function(i, e) {
            hljs.highlightBlock(e)
          });
        },
        onFailure: function(data) {
          console.log('Unable to Load SwaggerUI');
        },
        docExpansion: 'none',
        apisSorter: 'alpha',
        showRequestHeaders: false
      });

      window.swaggerUi.load();
    });
  </script>
</head>

<body class="swagger-section">
  <div id="message-bar" class="swagger-ui-wrap" data-sw-translate>&nbsp;</div>
  <div id="swagger-ui-container" class="swagger-ui-wrap"></div>
</body>

</html>
